/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2016 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: target_serial.c 301 2015-01-07 04:57:01Z ertl-ishikawa $
 */

/* #[<PREAMBLE>]#
 * #[<...>]# から #[</...>]# で囲まれたコメントは編集しないでください
 * tecsmerge によるマージに使用されます
 *
 * 属性アクセスマクロ #_CAAM_#
 * opnflg           bool_t           VAR_opnflg      
 *
 * 呼び口関数 #_TCPF_#
 * call port: ciSIOCBR signature: siSIOCBR context:task
 *   void           ciSIOCBR_readySend( );
 *   void           ciSIOCBR_readyReceive( );
 * call port: cInterruptRequest signature: sInterruptRequest context:task
 *   ER             cInterruptRequest_disable( );
 *   ER             cInterruptRequest_enable( );
 *
 * #[</PREAMBLE>]# */

/* プロトタイプ宣言や変数の定義をここに書きます #_PAC_# */
#include "tSIOPortTargetMain_tecsgen.h"
#include <t_syslog.h>
#include "tz1000.h"

#ifndef E_OK
#define	E_OK	0		/* success */
#define	E_ID	(-18)	/* illegal ID */
#endif

/* 受け口関数 #_TEPF_# */
/* #[<ENTRY_PORT>]# eSIOPort
 * entry port: eSIOPort
 * signature:  sSIOPort
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eSIOPort_open
 * name:         eSIOPort_open
 * global_name:  tSIOPortTargetMain_eSIOPort_open
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  シリアルI/Oポートのオープン
 */
/* SIOPCB *sio_opn_por(ID siopid, intptr_t exinf) */
void
eSIOPort_open(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	bool_t  opnflg;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
	/*
	 *  オープンしたポートがあるかをopnflgに読んでおく．
	 */
	opnflg = VAR_opnflg;

    if (!opnflg) {
        /*
         *  UART1ハードウェアの初期化はtarget_initialize
         *  で一部実施済み
         *  割込み要因マスク(UARTIMSC)は全禁止状態
         */
        /*
         * UART1_UARTIFLSの設定
         *   受信割込み頻度：受信FIFOのx%以上がFULL状態で割込みを起こす
         *     現状1/8に設定
         *   送信割込み頻度：送信FIFOのx%以上がEMPTY状態で割込みを起こす
         *     現状7/8に設定
         */
        sil_wrw_mem(UARTIFLS(UART1_BASE), (UARTIFLS_RX_1 | UARTIFLS_TX_14));
        /* 
         * 割込み要求のクリア
         * UART1_UARTICR の全要因をクリア
         */
        sil_wrw_mem(UARTICR(UART1_BASE), (UARTICR_TXIC | UARTICR_RXIC));
        while ((sil_rew_mem(UARTFR(UART1_BASE)) & UARTFR_TXFF) == UARTFR_TXFF);
        sil_wrw_mem(UARTDR(UART1_BASE), (uint32_t)('\0'));

        VAR_opnflg = true;
    }
}

/* #[<ENTRY_FUNC>]# eSIOPort_close
 * name:         eSIOPort_close
 * global_name:  tSIOPortTargetMain_eSIOPort_close
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  シリアルI/Oポートのクローズ
 */
/* void sio_cls_por(SIOPCB *p_siopcb) */
void
eSIOPort_close(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
	if (!VAR_opnflg) {
		cInterruptRequest_disable();
	}

}

/* #[<ENTRY_FUNC>]# eSIOPort_putChar
 * name:         eSIOPort_putChar
 * global_name:  tSIOPortTargetMain_eSIOPort_putChar
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  シリアルI/Oポートへの文字送信
 */
/* bool_t sio_snd_chr(SIOPCB *p_siopcb, char c) */
bool_t
eSIOPort_putChar(CELLIDX idx, char c)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
	if ((sil_rew_mem(UARTFR(UART1_BASE)) & UARTFR_TXFF) != UARTFR_TXFF) {
		/*
         *  送信FIFOがFullでないならば
		 *  DRに書き込む
		 */
        sil_wrw_mem(UARTDR(UART1_BASE), (uint32_t)((uint8_t)(c)));
		return(true);
	}
    else {
        return(false);
    }
}

/* #[<ENTRY_FUNC>]# eSIOPort_getChar
 * name:         eSIOPort_getChar
 * global_name:  tSIOPortTargetMain_eSIOPort_getChar
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  シリアルI/Oポートからの文字受信
 */
/* int_t sio_rcv_chr(SIOPCB *p_siopcb) */
int_t
eSIOPort_getChar(CELLIDX idx)
{
	CELLCB	*p_cellcb;
    int_t c = -1;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
	if ((sil_rew_mem(UARTFR(UART1_BASE)) & UARTFR_RXFE) != UARTFR_RXFE) {
		/*
         *  受信FIFOが空でないならば
		 *  DRを読む
		 */
        c = (int_t)(sil_rew_mem(UARTDR(UART1_BASE)) & 0xFF);
	}
	return(c);

}

/* #[<ENTRY_FUNC>]# eSIOPort_enableCBR
 * name:         eSIOPort_enableCBR
 * global_name:  tSIOPortTargetMain_eSIOPort_enableCBR
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  シリアルI/Oポートからのコールバックの許可
 */
/* void sio_ena_cbr(SIOPCB *p_siopcb, uint_t cbrtn) */
void
eSIOPort_enableCBR(CELLIDX idx, uint_t cbrtn)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
    switch (cbrtn) {
        case SIOReceiveReady:
            /* 受信割込みの許可 */
            sil_orw(UARTIMSC(UART1_BASE), UARTIMSC_RXIM);
            break;
        case SIOSendReady:
            /* 送信割込みの許可 */
            sil_orw(UARTIMSC(UART1_BASE), UARTIMSC_TXIM);
            break;
    }
}

/* #[<ENTRY_FUNC>]# eSIOPort_disableCBR
 * name:         eSIOPort_disableCBR
 * global_name:  tSIOPortTargetMain_eSIOPort_disableCBR
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  シリアルI/Oポートからのコールバックの禁止
 */
/* void sio_dis_cbr(SIOPCB *p_siopcb, uint_t cbrtn) */
void
eSIOPort_disableCBR(CELLIDX idx, uint_t cbrtn)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
    switch (cbrtn) {
        case SIOReceiveReady:
            /* 受信割込みの禁止 */
            sil_andw(UARTIMSC(UART1_BASE), ~UARTIMSC_RXIM);
            break;
        case SIOSendReady:
            /* 送信割込みの禁止 */
            sil_andw(UARTIMSC(UART1_BASE), ~UARTIMSC_TXIM);
            break;
    }
}

/* #[<ENTRY_PORT>]# eInitialize
 * entry port: eInitialize
 * signature:  sRoutineBody
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eInitialize_main
 * name:         eInitialize_main
 * global_name:  tSIOPortTargetMain_eInitialize_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  SIOドライバの初期化
 */
/* void sio_initialize(intptr_t exinf) */
void
eInitialize_main(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
    VAR_opnflg = false;

}

/* #[<ENTRY_PORT>]# eTerminate
 * entry port: eTerminate
 * signature:  sRoutineBody
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eTerminate_main
 * name:         eTerminate_main
 * global_name:  tSIOPortTargetMain_eTerminate_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eTerminate_main(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
}

/* #[<ENTRY_PORT>]# eiISRBody
 * entry port: eiISRBody
 * signature:  siHandlerBody
 * context:    non-task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eiISRBody_main
 * name:         eiISRBody_main
 * global_name:  tSIOPortTargetMain_eiISRBody_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  SIOの割込みサービスルーチン
 */
/* void sio_isr(intptr_t exinf) */
void
eiISRBody_main(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}
	else {
		/* エラー処理コードをここに記述します */
	} /* end if VALID_IDX(idx) */

	/* ここに処理本体を記述します #_TEFB_# */
    /* 
     * 割込み要求のクリア
     * UART1_UARTICR の全要因をクリア
     */
    sil_wrw_mem(UARTICR(UART1_BASE), (UARTICR_TXIC | UARTICR_RXIC));

	if ((sil_rew_mem(UARTFR(UART1_BASE)) & UARTFR_RXFE) != UARTFR_RXFE) {
		/*
         *  受信FIFOが空でないならば
		 *  受信通知コールバックルーチンを呼び出す．
		 */
		ciSIOCBR_readyReceive();
	}
	if ((sil_rew_mem(UARTFR(UART1_BASE)) & UARTFR_TXFF) != UARTFR_TXFF) {
		/*
         *  送信FIFOがFullでないならば
		 *  送信可能コールバックルーチンを呼び出す．
		 */
		ciSIOCBR_readySend();
	}
}

/* #[<POSTAMBLE>]#
 *   これより下に非受け口関数を書きます
 * #[</POSTAMBLE>]#*/
